@@ -27,7 +27,7 @@ __copyright__ = "(C) 2009 Science and Technology Facilities Council"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
 __license__ = "http://www.apache.org/licenses/LICENSE-2.0"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
-__revision__ = "$Id: xml.py 7130 2010-06-30 13:33:07Z pjkersha $"
+__revision__ = "$Id: xml.py 8009 2012-01-30 16:19:43Z rwilkinson $"
 from ndg.saml.xml import XMLConstants    
 
 
@@ -363,6 +363,20 @@ class SAMLConstants(XMLConstants):
     SAML2_SOAP11_BINDING_URI = "urn:oasis:names:tc:SAML:2.0:bindings:SOAP"
   
     
+    # SAML 2.0 profile for XACML
+
+    # SAML 2.0 profile for XACML assertion namespace
+    SAML2_XACML_ASSERTION_NS = "urn:oasis:xacml:2.0:saml:assertion:schema:os"
+
+    # SAML 2.0 profile for XACML assertion QName prefix
+    SAML2_XACML_ASSERTION_PREFIX = "xacml-saml"
+
+    # SAML 2.0 profile for XACML protocol namespace
+    SAML2_XACML_PROTOCOL_NS = "urn:oasis:xacml:2.0:saml:protocol:schema:os"
+
+    # SAML 2.0 profile for XACML protocol QName prefix
+    SAML2_XACML_PROTOCOL_PREFIX = "xacml-samlp"
+
 class QName(object):
     """XML Qualified Name
     @ivar __namespaceURI: the namespace the element is in
