@@ -7,7 +7,7 @@ __author__ = "P J Kershaw"
 __date__ = "15/02/10"
 __copyright__ = "(C) 2010 Science and Technology Facilities Council"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
-__revision__ = "$Id: queryinterface.py 7865 2011-02-02 14:40:21Z pjkersha $"
+__revision__ = "$Id: queryinterface.py 8009 2012-01-30 16:19:43Z rwilkinson $"
 __license__ = "http://www.apache.org/licenses/LICENSE-2.0"
 import logging
 log = logging.getLogger(__name__)
@@ -22,13 +22,25 @@ from ndg.soap.etree import SOAPEnvelope
 from ndg.saml.utils import str2Bool
 from ndg.saml.utils.factory import importModuleObject
 from ndg.saml.xml import UnknownAttrProfile
+from ndg.saml.xml.etree import QName
 from ndg.saml.common import SAMLVersion
 from ndg.saml.utils import SAMLDateTime
 from ndg.saml.saml2.core import (Response, Status, StatusCode, StatusMessage, 
                                  Issuer) 
 from ndg.saml.saml2.binding.soap import SOAPBindingInvalidResponse
 
-
+try:
+    from ndg.saml.saml2.xacml_profile import XACMLAuthzDecisionQuery
+    import ndg.saml.xml.etree_xacml_profile as etree_xacml_profile
+except ImportError, e:
+    from warnings import warn
+    warn('Error importing XACML packages - disabling SAML XACML profile ' + \
+         'support.  (Error is: %s)' % e)
+    class XACMLAuthzDecisionQuery(object):
+        """XACML Authz Decision Query substitute"""
+        DEFAULT_ELEMENT_LOCAL_NAME = 'XACMLAuthzDecisionQuery'
+        
+    
 class SOAPQueryInterfaceMiddlewareError(Exception):
     """Base class for WSGI SAML 2.0 SOAP Query Interface Errors"""
 
@@ -64,6 +76,7 @@ class SOAPQueryInterfaceMiddleware(SOAPMiddleware):
     RESPONSE_ENVELOPE_CLASS_OPTNAME = 'responseEnvelopeClass'
     SERIALISE_OPTNAME = 'serialise'
     DESERIALISE_OPTNAME = 'deserialise' 
+    DESERIALISE_XACML_PROFILE_OPTNAME = 'deserialiseXacmlProfile'
     SAML_VERSION_OPTNAME = 'samlVersion'
     ISSUER_NAME_OPTNAME = 'issuerName'
     ISSUER_FORMAT_OPTNAME = 'issuerFormat'
@@ -77,6 +90,7 @@ class SOAPQueryInterfaceMiddleware(SOAPMiddleware):
         RESPONSE_ENVELOPE_CLASS_OPTNAME,
         SERIALISE_OPTNAME,
         DESERIALISE_OPTNAME,
+        DESERIALISE_XACML_PROFILE_OPTNAME,
         SAML_VERSION_OPTNAME,
         ISSUER_NAME_OPTNAME,
         ISSUER_FORMAT_OPTNAME,
@@ -99,6 +113,7 @@ class SOAPQueryInterfaceMiddleware(SOAPMiddleware):
         self.__responseEnvelopeClass = None
         self.__serialise = None
         self.__deserialise = None
+        self.__deserialiseXacmlProfile = None
         self.__issuer = None
         self.__clockSkewTolerance = timedelta(seconds=0.)
         self.__verifyTimeConditions = True
@@ -170,6 +185,24 @@ class SOAPQueryInterfaceMiddleware(SOAPMiddleware):
                            doc="callable to de-serialise response from XML "
                                "type")        
 
+    def _getDeserialiseXacmlProfile(self):
+        return self.__deserialiseXacmlProfile
+
+    def _setDeserialiseXacmlProfile(self, value):
+        if isinstance(value, basestring):
+            self.__deserialiseXacmlProfile = importModuleObject(value)
+
+        elif callable(value):
+            self.__deserialiseXacmlProfile = value
+        else:
+            raise TypeError('Expecting callable for "deserialiseXacmlProfile"; '
+                            'got %r' % value)
+
+    deserialiseXacmlProfile = property(_getDeserialiseXacmlProfile,
+                                       _setDeserialiseXacmlProfile,
+                                       doc="callable to de-serialise response "
+                                       "from XML type with XACML profile")
+
     def _getIssuer(self):
         return self.__issuer
 
@@ -399,8 +432,14 @@ class SOAPQueryInterfaceMiddleware(SOAPMiddleware):
         samlResponse = self._initResponse()
         
         try:
-            samlQuery = self.deserialise(queryElem)
-            
+            queryType = QName.getLocalPart(queryElem.tag)
+            if queryType == XACMLAuthzDecisionQuery.DEFAULT_ELEMENT_LOCAL_NAME:
+                # Set up additional ElementTree parsing for XACML profile.
+                etree_xacml_profile.setElementTreeMap()
+                samlQuery = self.deserialiseXacmlProfile(queryElem)
+            else:
+                samlQuery = self.deserialise(queryElem)
+
         except UnknownAttrProfile, e:
             log.exception("%r raised parsing incoming query: %s" % 
                           (type(e), traceback.format_exc()))
